/*  tests to see if the libgda crashes on a SQL Error
 *  example:  "select * from NoSuchTable"
*/
#include <stdlib.h>
#include "errorcrash.h"

void
do_tests (void)
{
        GdaClient *client; 
	GdaConnection *cnc;

        /* connection data */
	gchar *data_source_name = "dsn20";     
	gchar *username         = "danmorg";
	gchar *password         = "viewsonic";

        /* get new gda client */
        g_print("gda_client_new\n");
	client = gda_client_new ();

	g_return_if_fail (GDA_IS_CLIENT (client));

        /* attempt to connect to database */
        g_print("gda_client_open_connection\n");
	cnc = gda_client_open_connection (client, data_source_name, username, password);

        /* check to see if connection was successful */
	if (!GDA_IS_CONNECTION (cnc)) {
                /* connection was unsuccessfull */
		g_print ("\n ** CONNECTION ERROR: could not open connection to %s\n", data_source_name);
		g_object_unref (G_OBJECT (client));
		return;
	}
	else {
                /* connection was a success */
		g_print ("\n ** CONNECTION SUCCESS: connection to database was successful.\n");
	}

	execute_sql(client, cnc, 1, "select * from pg_tables");
	execute_sql(client, cnc, 2, "select * from pg_tables;");
	execute_sql(client, cnc, 3, "select * from pg_tables ");
	execute_sql(client, cnc, 4, "select * from pg_tables ;");
	execute_sql(client, cnc, 5, "select * from pg_tables; ");
	execute_sql(client, cnc, 6, "select * from pg_tables;select * from pg_views");
	execute_sql(client, cnc, 7, "select * from pg_tables;\nselect * from pg_views");
	execute_sql(client, cnc, 8, "select * from nosuchtable");
        
	/* close the connection */
	gda_connection_close (cnc);

        /* free the client */
	g_object_unref (G_OBJECT (client));
}

void
execute_sql(GdaClient *client, GdaConnection *cnc, int testno, const gchar *text)
{
	GdaCommand *select_command;
	GList *list; /* recordset or model */

        g_print("\n------------------------------------\n");
	g_print("SQL Test %d = \"%s\"\n", testno, text);
   
        g_print("  gda_command_new\n") ;
        /* get a new instance of command and set the SQL statement to command */
	select_command = gda_command_new ( text,
			GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);

        g_print("  gda_connection_execute_command\n");
        /* execute the command */
	list = gda_connection_execute_command (cnc, select_command, NULL);

        g_print("  g_list_foreach display_recordset\n");
        /* for each record in the recordset, call function display_recordset */
	g_list_foreach (list, (GFunc) display_recordset_data, NULL);

      
        /* free each record in the recordset */
	g_print("  g_list_foreach o_object_unref\n");
	g_list_foreach (list, (GFunc) g_object_unref, NULL);

        /* free the recordset */
	g_print("  g_list_free list\n");
	g_list_free (list);
        
        /* free the command */ 
	g_print("  gda_command_free\n");
        gda_command_free (select_command);

	g_print("*** No segfault here!\n");
}

void
display_recordset_data (GdaDataModel *model)
{
	gint cols, rows;
	gint c, r;

	g_return_if_fail (GDA_IS_RECORDSET (model));

	cols = gda_data_model_get_n_columns (model);
	rows = gda_data_model_get_n_rows (model);

	g_print (" Displaying recordset %p, with %d columns and %d rows:\n",
		 model, cols, rows);

        /* limit the results to just one row and one column */
	for (r = 0; r < rows && r < 1; r++) {

		for (c = 0; c < cols && c < 1; c++) {
			GdaValue *value;
			gchar *strvalue;
			const gchar *colname;

			colname = gda_data_model_get_column_title (model, c);
			value = (GdaValue *) gda_data_model_get_value_at (model, c, r);
			strvalue = gda_value_stringify (value);
			g_print ("  Row: %d  Colname: %s  Value: ('%s')\n", 
				r, colname, strvalue);
			g_free (strvalue);
		}
	}
}

static void
run_gda_test (gpointer user_data)
{
	do_tests();

	gda_main_quit ();
}

int
main(int argc, char *argv[])
{
	gda_init ("errorcrash", VERSION, argc, argv);

	gda_main_run ((GdaInitFunc) run_gda_test, NULL);

	return (0);
}


